use leptos::{html::Input, *};
use web_sys::File;

use super::ResultMessage;
use crate::{
    components::ResultMessageData, error::ReadKleError, models::Colorway, utils::read_file,
};

#[component]
pub fn KeyboardFromFile(
    cx: Scope,
    set_keyboard: WriteSignal<Option<kle_serial::Keyboard>>,
) -> impl IntoView {
    let file_input = create_node_ref::<Input>(cx);
    let (file, set_file) = create_signal(cx, Option::<File>::None);
    let (result_message, set_result_message) = create_signal(cx, Option::<ResultMessageData>::None);
    let on_submit = move || {
        spawn_local(async move {
            match read_kle_from_file(file).await {
                Ok(keyboard) => {
                    set_result_message(Some(ResultMessageData {
                        title: "Success".to_owned(),
                        message: view! { cx,
                            "Loaded KLE layout "<b>{&keyboard.metadata.name}</b>" successfully!"
                        }
                        .into_view(cx),
                        colorway: Colorway::Ok,
                    }));
                    set_keyboard(Some(keyboard));
                }
                Err(err) => {
                    set_result_message(Some(ResultMessageData {
                        message: view! { cx,
                            {
                                err.to_string()
                            }
                        }
                        .into_view(cx),
                        title: <ReadKleError as Into<&str>>::into(err).to_string(),
                        colorway: Colorway::Bad,
                    }));
                    file_input().unwrap().set_value("");
                }
            }
        });
    };
    view! { cx,
        <h3>"Load KLE JSON from file"</h3>
        <ResultMessage message=result_message />
        <input
            type="file"
            accept="application/json"
            node_ref=file_input
            on:change=move |_| {
                set_file(file_input().unwrap().files().and_then(|files| files.get(0)));
                on_submit();
            }
        />
    }
}

async fn read_kle_from_file(
    file: ReadSignal<Option<File>>,
) -> Result<kle_serial::Keyboard, ReadKleError> {
    Ok(serde_json::from_str(&read_file(file).await?)?)
}
