use class_list::class_list;
use leptos::*;

use crate::models::Colorway;

#[derive(Clone)]
pub struct ResultMessageData {
    pub title: String,
    pub message: View,
    pub colorway: Colorway,
}

#[component]
pub fn ResultMessage(cx: Scope, message: ReadSignal<Option<ResultMessageData>>) -> impl IntoView {
    let (open, set_open) = create_signal(cx, true);
    create_effect(cx, move |_| {
        message.track();
        set_open(true);
    });
    move || {
        view! { cx,
            <Show when=open fallback=|_| ()>
                {move || message().map(|ResultMessageData { title, message, colorway }| view! { cx,
                    <div class=class_list!["box", colorway] style="position: relative">
                        <strong class="block titlebar">{title}</strong>
                        <button class="iconbutton" on:click=move |_| set_open(false) style="position: absolute; bottom: 0.5em; right: 0.5em">"×"</button>
                        <p>{message}</p>
                    </div>
                })}
            </Show>
        }
    }
}
