use derive_more::From;
use strum_macros::IntoStaticStr;

use super::FetchFileError;

#[derive(From, IntoStaticStr)]
pub enum FetchKleError {
    #[strum(
        serialize = "Invalid source. Must be KLE layout, gist URL, gist ID, or direct URL to JSON file"
    )]
    InvalidSource,
    #[strum(serialize = "Failed to fetch file")]
    FetchFile(FetchFileError),
    #[strum(serialize = "Failed to parse KLE JSON")]
    Serde(serde_json::Error),
}

impl ToString for FetchKleError {
    fn to_string(&self) -> String {
        use FetchKleError::*;
        match self {
            FetchFile(error) => Some(match error {
                FetchFileError::Request(error_info) => {
                    let mut full_error = <&FetchFileError as Into<&str>>::into(error).to_string();
                    if let Some(error_info) = error_info.as_string() {
                        full_error.push_str(": ");
                        full_error.push_str(&error_info);
                    }
                    full_error
                }
                FetchFileError::ReadToString => error.to_string(),
            }),
            Serde(error) => Some(error.to_string()),
            _ => None,
        }
        .unwrap_or_else(|| "".to_string())
    }
}
